@@ -1,6 +1,7 @@
 '''
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -15,15 +16,25 @@ License for the specific language governing permissions and limitations
 under the License.
 '''
 
-from root import RTSRoot
-from utils import RTSLibError, RTSLibBrokenLink, RTSLibNotInCFS
+if __name__ == "rtslib":
+    from warnings import warn
+    warn("'rtslib' package name for rtslib-fb is deprecated, please"
+         + " instead import 'rtslib_fb'", UserWarning, stacklevel=2)
 
-from target import LUN, MappedLUN
-from target import NodeACL, NetworkPortal, TPG, Target
-from fabric import FabricModule
+from .root import RTSRoot
+from .utils import RTSLibError, RTSLibBrokenLink, RTSLibNotInCFS
+from .utils import RTSLibALUANotSupported
 
-from tcm import FileIOStorageObject, BlockStorageObject
-from tcm import PSCSIStorageObject, RDMCPStorageObject
+from .target import LUN, MappedLUN
+from .target import NodeACL, NetworkPortal, TPG, Target
+from .target import NodeACLGroup, MappedLUNGroup
+from .fabric import FabricModule
+
+from .tcm import FileIOStorageObject, BlockStorageObject
+from .tcm import PSCSIStorageObject, RDMCPStorageObject, UserBackedStorageObject
+from .tcm import StorageObjectFactory
+
+from .alua import ALUATargetPortGroup
 
 __version__ = 'GIT_VERSION'
 __author__ = "Jerome Martin <jxm@risingtidesystems.com>"
